// Oppgave ? fra NM/SM i programmering 2002
// Lst av Dag Langmyhr (dag@ifi.uio.no).

import java.io.*;

class Scanner {
    public char sy;
    private BufferedReader f;
    private static final boolean debug = false;

    public Scanner () throws IOException {
	f = new BufferedReader(new InputStreamReader(System.in));  next();
    }

    public void next () {
	try {
	    do {
		int c = f.read();
		sy = c<0 ? '?' : (char)c;
	    } while (sy!='(' && sy!=')' && sy!='.' && sy!='?' &&
		     ! Character.isLetter(sy));
	} catch (IOException e) {
	    System.err.println("Read error!");  System.exit(1);
	}
	if (debug)
	    System.err.println("DEBUG> Scanner: sy='" + sy + "'");
    }

    public void test (char c) {
	if (sy == c) return;
	System.err.println("A '" + c + "' was expected; a '" + 
			   sy + "' was found.");
	System.exit(2);
    }
}


abstract class Expression {
    private static final boolean debug = false;
    public static final int maxCycles = 1000;

    public boolean lastExpr () { return false; }

    public Expression eval (int cycles) { 
	if (debug && cycles<50) {
	    System.err.print("DEBUG> ");  print(System.err);
	    System.err.println(" evaluates to itself.");
	}
	return this; 
    }

    public abstract void print (PrintStream p);

    public static Expression read (Scanner s) {
	if (s.sy == 'L') return lambda.read(s);
	if (s.sy == '(') return FuncCall.read(s);
	return Variable.read(s);
    }

    public Expression replace (char v, Expression e) { return this; }
}


class Undefined extends Expression {
    public void print (PrintStream p) { p.print("unterminated"); }
}


class Variable extends Expression {
    char var;

    public Variable (char v) { var = v; }

    public boolean lastExpr () { return var == 'z'; }

    public void print (PrintStream p) { p.print(var); }

    public static Expression read (Scanner s) {
	char v = s.sy;  
	s.next();
	return new Variable(v);
    }

    public Expression replace (char v, Expression e) {
	return  var==v ? e : this;
    }
}


class lambda extends Expression {
    char var;  
    Expression expr;
    private static final boolean debug = false;

    public lambda (char v, Expression e) { var = v;  expr = e; }

    public void print (PrintStream p) {
	p.print("L" + var + ".");  expr.print(p);
    }

    public static Expression read (Scanner s) {
	s.next();  
	char v = s.sy;
	s.next();  s.test('.');  s.next();
	Expression e = Expression.read(s);
	return new lambda(v,e);
    }

    public Expression replace (char v, Expression e) {
	Expression r = var==v ? this : new lambda(var,expr.replace(v,e));
	if (debug) {
	    System.err.print("DEBUG> Replacing " + v + " with ");
	    e.print(System.err);  System.err.print(" in ");
	    print(System.err);  System.err.print(" gives ");
	    r.print(System.err);  System.err.println();
	}
	return r;
    }

    public static void main (String arg[]) throws Exception {
	Scanner s = new Scanner();
	while (true) {
	    Expression e = Expression.read(s);
	    if (debug) {
		System.err.print("DEBUG> Read ");  e.print(System.err);
		System.err.println();
	    }
	    for (int i = 1;  i <= maxCycles;  ++i) {
		e = e.eval(0);
		if (!(e instanceof FuncCall)) break;
	    }
	    e.print(System.out);  System.out.println();
	    if (e.lastExpr()) break;
	}
    }
}


class FuncCall extends Expression {
    Expression func, param;
    private static final boolean debug = false;

    public FuncCall (Expression f, Expression p) { func = f;  param = p; }

    public Expression eval (int cycles) {
	if (cycles > maxCycles) return new Undefined();

	Expression f = func.eval(cycles+1), p = param.eval(cycles+1);
	if (!(f instanceof lambda)) return new FuncCall(f,p);

	lambda l = (lambda)f;
	Expression fe = l.expr.replace(l.var,p).eval(cycles+1);
	if (debug && cycles<50) {
	    System.err.print("DEBUG> ");  new FuncCall(f,p).print(System.err);
	    System.err.print(" evaluates to ");  fe.print(System.err);
	    System.err.println();
	}
	return fe; 
    }

    public void print (PrintStream p) {
	p.print("(");  func.print(p);  p.print(")");  param.print(p);
    }

    public static Expression read (Scanner s) {
	s.next();
	Expression f = Expression.read(s);
	s.test(')');  s.next();
	Expression p = Expression.read(s);
	return new FuncCall(f,p);
    }

    public Expression replace (char v, Expression e) {
	return new FuncCall(func.replace(v,e),param.replace(v,e));
    }
}
